
#import "ViewController.h"

@implementation ViewController {
    LabelPrinterSDK *sdk;
    NSMutableArray *deviceList;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    sdk = [SharedLabelPrinterSDK sharedInstance];
    deviceList = [NSMutableArray array];
    
    // Set UITextView border
    _tvHistory.layer.borderColor = UIColor.lightGrayColor.CGColor;
    _tvHistory.layer.borderWidth = 1;
    _tvHistory.layer.cornerRadius = 8;
    
    // Set UITableView Border
    _tblDeviceList.layer.borderColor = UIColor.lightGrayColor.CGColor;
    _tblDeviceList.layer.borderWidth = 1;
    _tblDeviceList.layer.cornerRadius = 8;
    
    // SDK Open
    SDK_RESULT_CODES result = [sdk open];
    
    if (result != SDK_RESULT_SUCCESS) {
        [self addHistoryWithText:@"SDK Open error."];
        return;
    }
    
    sdk.delegate = self;
}

- (IBAction)onChangeInterfaceType:(UISegmentedControl *)sender {
    [sdk cancelToLookupPrinter];
    [deviceList removeAllObjects];
    [_tblDeviceList reloadData];
}

- (IBAction)onPrinterLookup:(UIButton *)sender {
    [sdk cancelToLookupPrinter];
    [deviceList removeAllObjects];
    [_tblDeviceList reloadData];
    
    if (_segInterfaceType.selectedSegmentIndex == 1) {
        [deviceList addObjectsFromArray:[sdk getPairedDevices]];
        [_tblDeviceList reloadData];
    } else {
        [sdk lookupPrinterWithCount:3 interval:1000];
    }
}

- (IBAction)onConnect:(UIButton *)sender {
    if ([sdk isConnected]) {
        [sdk disconnect];
    }
    
    SDK_RESULT_CODES result;
    
    switch (_segInterfaceType.selectedSegmentIndex) {
        case 0:
            result = [sdk connectWithAddress:_tfAddress.text port:@"9100"];
            break;
        case 1:
            result = [sdk connectWithSerialNumber:_tfAddress.text];
            break;
        case 2:
            result = [sdk connectWithAddress:_tfAddress.text port:@""];
            break;
        default:
            [self addHistoryWithText:@"Invalidate interfaceType."];
            return;
    }
    
    [self addHistoryWithText:[NSString stringWithFormat:@"Printer connect %@", result == SDK_RESULT_SUCCESS ? @"success" : @"fail"]];
}

- (void) addHistoryWithText:(NSString *)text {
    _tvHistory.text = [_tvHistory.text stringByAppendingString:[text stringByAppendingString:@"\r\n"]];
    [_tvHistory scrollRangeToVisible:NSMakeRange(_tvHistory.text.length - 1, 1)];
}

- (IBAction)onDisconnect:(UIButton *)sender {
    
    //    This process is to confirm that the data has been completely transferred.
    //    It is not necessary if you do not verify the data transfer.
    [sdk checkPrinterStatus:^(NSInteger status) {
        
        // disconnect
        SDK_RESULT_CODES result = [self->sdk disconnect];
        [self addHistoryWithText:[NSString stringWithFormat:@"Printer disconnect %@", result == SDK_RESULT_SUCCESS ? @"success" : @"fail"]];
        
    }];

//    __SDK_RESULT_CODES_ result = [sdk disconnect];
//    [self addHistoryWithText:[NSString stringWithFormat:@"Printer disconnect %@", result == _SDK_RESULT_SUCCESS ? @"success" : @"fail"]];
}

- (IBAction)onPrint:(UIButton *)sender {
    if (![sdk isConnected]) {
        [self addHistoryWithText:@"Not connected"];
        return;
    }
    
    [self performSegueWithIdentifier:@"PrintTestSegue" sender:nil];
}

- (IBAction)onClear:(UIButton *)sender {
    _tvHistory.text = @"";
}

#pragma mark - LabelPrinterSDKDelegate Override
- (void)willConnect:(LabelPrinterSDK *)controller
            printer:(LabelPrinterObject *)printer {
    [self addHistoryWithText:@"Delegate : willConnect"];
}

- (void)didConnect:(LabelPrinterSDK *)controller
           printer:(LabelPrinterObject *)printer {
    [self addHistoryWithText:@"Delegate : didConnect"];
}

- (void)didNotConnect:(LabelPrinterSDK *)controller
              printer:(LabelPrinterObject *)printer
            withError:(NSError *)error {
    [self addHistoryWithText:@"Delegate : didNotConnect"];
}

- (void)didDisconnect:(LabelPrinterSDK *)controller
              printer:(LabelPrinterObject *)printer {
    [self addHistoryWithText:@"Delegate : didDisconnect"];
}

- (void)didBeBrokenConnection:(LabelPrinterSDK *)controller
                      printer:(LabelPrinterObject *)printer
                    withError:(NSError *)error {
    [self addHistoryWithText:@"Delegate : didBeBrokenConnection"];
}

- (void)willLookupPrinter:(LabelPrinterSDK *)controller {
    [self addHistoryWithText:@"Delegate : willLookupPrinter"];
}

- (void)didFindPrinter:(LabelPrinterSDK *)controller
               printer:(LabelPrinterObject *)printer {
    [self addHistoryWithText:@"Delegate : didFindPrinter"];
}

- (void)didLookupPrinters:(LabelPrinterSDK *)controller
              printerList:(NSArray *)printerList {
    [self addHistoryWithText:@"Delegate : didLookupPrinters"];

    NSInteger segIdx = _segInterfaceType.selectedSegmentIndex;
    
    
    NSPredicate *predicate = [NSPredicate predicateWithBlock:^BOOL(id  _Nullable evaluatedObject, NSDictionary<NSString *,id> * _Nullable bindings) {
        LabelPrinterObject *obj = evaluatedObject;
        SDK_INTERFACE_TYPE type = [obj getInterfaceType];
        BOOL result = NO;
        switch (segIdx) {
            case 0: //Network
                result = type == SDK_INTERFACE_TYPE_WIFI || type == SDK_INTERFACE_TYPE_ETHERNET;
                break;
            case 1: //Bluetooth
                result = type == SDK_INTERFACE_TYPE_BLUETOOTH;
                break;
            case 2:
                result = type == SDK_INTERFACE_TYPE_BLUETOOTH_BLE;
                break;
            default:
                break;
        }
        
        return result;
    }];
    
    [deviceList addObjectsFromArray:[printerList filteredArrayUsingPredicate:predicate]];
    [_tblDeviceList reloadData];
}

- (void)canNotFoundPrinter {
    [self addHistoryWithText:@"Delegate : canNotFoundPrinter"];
}

#pragma mark - UITextFieldDelegate
- (BOOL)textFieldShouldReturn:(UITextField *)textField {
    [textField resignFirstResponder];
    return YES;
}

#pragma mark - UITableViewDelegate
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    if (indexPath.row > deviceList.count) {
        return;
    }
    
    LabelPrinterObject *device = [deviceList objectAtIndex:indexPath.row];
    
    switch (_segInterfaceType.selectedSegmentIndex) {
        case 0:
            _tfAddress.text = [device getAddress]; break;
        case 1:
            _tfAddress.text = [device getSerialNumber]; break;
        case 2:
            _tfAddress.text = [device getMacAddress]; break;
        default:
            break;
    }
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return [deviceList count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"deviceCell"];
    UILabel *lblDevice = [cell viewWithTag:10];
    
    if (deviceList.count < indexPath.row) {
        return [UITableViewCell new];
    }
    
    LabelPrinterObject *object = [deviceList objectAtIndex:indexPath.row];
    SDK_INTERFACE_TYPE interfaceType = [object getInterfaceType];

    NSString *text = nil;
    switch (interfaceType) {
        case SDK_INTERFACE_TYPE_ETHERNET:
        case SDK_INTERFACE_TYPE_WIFI:
            text = [NSString stringWithFormat:
                    @"Address : %@\r\n"
                    "Port : %@", [object getAddress], [object getPort]];
            break;
        case SDK_INTERFACE_TYPE_BLUETOOTH:
            text = [NSString stringWithFormat:
                    @"Device Name : %@\r\n"
                    "Serial : %@", [object getBluetoothDeviceName], [object getSerialNumber] ];
            break;
        case SDK_INTERFACE_TYPE_BLUETOOTH_BLE:
            text = [NSString stringWithFormat:
                    @"MacAddress : %@", [object getMacAddress]
                    ];
            break;
        default:
            text = @"";
            break;
    }
    
    lblDevice.text = text;
    
    return cell;
}

@end
