
import UIKit

class ViewController: BaseViewController {

    @IBOutlet weak var segInterfaceType: UISegmentedControl!
    @IBOutlet weak var tfAddress: UITextField!
    @IBOutlet weak var tblDeviceList: UITableView!
    @IBOutlet weak var tvHistory: UITextView!
    
    let sdk = SharedLabelPrinterSDK.sdk
    var deviceList:[LabelPrinterObject] = []
    
    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view.
        
        // Set UITextView border
        tvHistory.layer.borderColor = UIColor.lightGray.cgColor
        tvHistory.layer.borderWidth = 1
        tvHistory.layer.cornerRadius = 8
        
        // Set UITableView Border
        tblDeviceList.layer.borderColor = UIColor.lightGray.cgColor
        tblDeviceList.layer.borderWidth = 1
        tblDeviceList.layer.cornerRadius = 8
        
        // SDK Open
        let result = sdk.open()
        
        guard result == .SUCCESS else {
            addHistory(text: "SDK Open error.")
            return
        }
        
        LabelPrinterSDK.setEnableLog(true, saveFile: true, hexDump: false)
        
        sdk.delegate = self
    }
    
    @IBAction func onChangeInterfaceType(_ sender: UISegmentedControl) {
        sdk.cancelToLookupPrinter()
        deviceList.removeAll()
        tblDeviceList.reloadData()
    }
    
    @IBAction func onPrinterLookup(_ sender: UIButton) {
        sdk.cancelToLookupPrinter()
        deviceList.removeAll()
        tblDeviceList.reloadData()
        
        if segInterfaceType.selectedSegmentIndex == 1,
            let paired = sdk.getPairedDevices() as? [LabelPrinterObject] {
            deviceList.append(contentsOf: paired)
            tblDeviceList.reloadData()
        } else {
            sdk.lookupPrinter(withCount: 3, interval: 1000)
        }

    }
    
    @IBAction func onConnect(_ sender: UIButton) {
        if sdk.isConnected() {
            sdk.disconnect()
        }
        
        let result:SDK_RESULT_CODES
        
        switch segInterfaceType.selectedSegmentIndex {
        case 0:
            result = sdk.connect(withAddress: tfAddress.text, port: "9100")
        case 1:
            result = sdk.connect(withSerialNumber: tfAddress.text)
        case 2:
            result = sdk.connect(withAddress: tfAddress.text, port: "")
        default:
            addHistory(text: "Invalidate interfaceType.")
            return
        }
        
        addHistory(text: "Printer connect \(result == .SUCCESS ? "success" : "fail")")
    }
    
    func addHistory(text:String) {
        tvHistory.text = tvHistory.text.appending(text.appending("\r\n"))
        tvHistory.scrollRangeToVisible(NSMakeRange(tvHistory.text.count - 1, 1))
    }
    
    @IBAction func onDisconnect(_ sender: UIButton) {
        let result:SDK_RESULT_CODES
        
        //  Check the completion of data transmission and then close
        result = sdk.disconnect(withTimeout: 10)
        self.addHistory(text: "Printer disconnect \(result == .SUCCESS ? "success" : "fail")")
        
        //  Disconnect regardless of data transfer
//        result = sdk.disconnect()
//        self.addHistory(text: "Printer disconnect \(result == .SDK_RESULT_SUCCESS ? "success" : "fail")")
    }
    
    @IBAction func onPrint(_ sender: UIButton) {
        guard sdk.isConnected() else {
            addHistory(text: "Not connected")
            return
        }
        
        self.performSegue(withIdentifier: "PrintTestSegue", sender: nil)
    }
    
    @IBAction func onClear(_ sender: UIButton) {
        tvHistory.text = ""
    }
}

// MARK: - LabelPrinterSDKDelegate Override
extension ViewController {
    override func willConnect(_ controller: LabelPrinterSDK!, printer: LabelPrinterObject!) {
        addHistory(text: "Delegate : willConnect")
    }
    
    override func didConnect(_ controller: LabelPrinterSDK!, printer: LabelPrinterObject!) {
        addHistory(text: "Delegate : didConnect")
    }
    
    override func didNotConnect(_ controller: LabelPrinterSDK!, printer: LabelPrinterObject!, withError error: Error!) {
        addHistory(text: "Delegate : didNotConnect")
    }
    
    override func didDisconnect(_ controller: LabelPrinterSDK!, printer: LabelPrinterObject!) {
        addHistory(text: "Delegate : didDisconnect")
    }
    
    override func didBeBrokenConnection(_ controller: LabelPrinterSDK!, printer: LabelPrinterObject!, withError error: Error!) {
        addHistory(text: "Delegate : didBeBrokenConnection")
    }
    
    override func willLookupPrinter(_ controller: LabelPrinterSDK!) {
        addHistory(text: "Delegate : willLookupPrinter")
    }
    
    override func didFindPrinter(_ controller: LabelPrinterSDK!, printer: LabelPrinterObject!) {
        addHistory(text: "Delegate : didFindPrinter")
    }
    
    override func didLookupPrinters(_ controller: LabelPrinterSDK!, printerList: [Any]!) {
        addHistory(text: "Delegate : didLookupPrinters")
        
        if let printerList = printerList as? [LabelPrinterObject] {
            
            let filteredList = printerList.filter { (obj) -> Bool in
                let type = UInt(obj.getInterfaceType())
                let result:Bool
                switch segInterfaceType.selectedSegmentIndex {
                case 0: // Network
                    result = type == SDK_INTERFACE_TYPE.WIFI.rawValue ||
                        type == SDK_INTERFACE_TYPE.ETHERNET.rawValue
                case 1:
                    result = type == SDK_INTERFACE_TYPE.BLUETOOTH.rawValue
                case 2:
                    result = type == SDK_INTERFACE_TYPE.BLUETOOTH_BLE.rawValue
                default:
                    result = false
                }
                
                return result
            }
            
            deviceList.append(contentsOf: filteredList)
            tblDeviceList.reloadData()
        }
    }
    
    override func canNotFoundPrinter() {
        addHistory(text: "Delegate : canNotFoundPrinter")
    }

}

extension ViewController : UITextViewDelegate {

}

// MARK: - UITableViewDelegate
extension ViewController : UITableViewDelegate {
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        guard indexPath.row < deviceList.count else {
            return
        }
        
        let device = deviceList[indexPath.row]
        
        switch segInterfaceType.selectedSegmentIndex {
        case 0:
            tfAddress.text = device.getAddress()
        case 1:
            tfAddress.text = device.getSerialNumber()
        case 2:
            tfAddress.text = device.getMacAddress()
        default:
            break
        }
        
        tableView.selectRow(at: nil, animated: false, scrollPosition: .none)
    }
}

// MARK: - UITableViewDataSource
extension ViewController : UITableViewDataSource {
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return deviceList.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        guard let cell = tableView.dequeueReusableCell(withIdentifier: "deviceCell"),
            let lblDevice = cell.viewWithTag(10) as? UILabel else {
            return UITableViewCell()
        }
        
        guard deviceList.count > indexPath.row else {
            return UITableViewCell()
        }
        
        let object = deviceList[indexPath.row]
        let interfaceType = object.getInterfaceType()

        let text:String
        
        switch interfaceType {
        case SDK_INTERFACE_TYPE.ETHERNET.rawValue: fallthrough
        case SDK_INTERFACE_TYPE.WIFI.rawValue:
            text = """
            Address : \(object.getAddress() ?? "")
            Port : \(object.getPort() ?? "")
            """
        case SDK_INTERFACE_TYPE.BLUETOOTH.rawValue:
            text = """
            Device Name : \(object.getModelName() ?? "")
            Serial : \(object.getSerialNumber() ?? "")
            """
        case SDK_INTERFACE_TYPE.BLUETOOTH_BLE.rawValue:
            text = """
            MacAddress : \(object.getMacAddress() ?? "")
            """
        default:
            text = ""
        }
        
        lblDevice.text = text
        
        return cell
    }
}
